var global = (function (){return (function(){return this;}).call(null);})();
(function(Nuvola)
{
    'use strict';
// Included file '/builddir/build/BUILD/nuvolaruntime-4.11.0/src/mainjs/main.js'
Nuvola.global = global
try {
  Nuvola.window = window // ReferenceError with bare Global Object
  Nuvola.checkMSE = function () {
    if (!window.MediaSource) {
      throw new Error('Media Source Extension is not supported.')
    }
  }
} catch (e) {
  global.window = null
}
// Included file '/builddir/build/BUILD/nuvolaruntime-4.11.0/src/mainjs/prototype.js'
var $prototype = function (proto, mixins) {
  if (proto === undefined) { throw new Error('Proto argument must be specified. Can be null.') }
  var object = Object.create(proto)
  var len = arguments.length
  for (var i = 1; i < len; i++) {
    var mixin = arguments[i]
    for (var name in mixin) { object[name] = mixin[name] }
  }
  return object
}
var $object = function (proto, initArgs) {
  if (proto === undefined) { throw new Error('Proto argument must be specified. Can be null.') }
  var object = Object.create(proto)
  if (object.$init) { object.$init.apply(object, [].slice.call(arguments, 1)) }
  return object
}
Nuvola.$object = $object
Nuvola.$prototype = $prototype
// Included file '/builddir/build/BUILD/nuvolaruntime-4.11.0/src/mainjs/signals.js'
var SignalsMixin = {}
SignalsMixin.addSignal = function (name) {
  if (this.signals == null) { this.signals = {} }
  this.signals[name] = []
}
SignalsMixin.registerSignals = function (signals) {
  if (this.signals === undefined) { this.signals = {} }
  var size = signals.length
  for (var i = 0; i < size; i++) {
    this.signals[signals[i]] = []
  }
}
SignalsMixin.connect = function (name, object, handlerName) {
  handlerName = handlerName || ('_on' + name)
  var handlers = this.signals[name]
  if (handlers === undefined) { throw new Error("Unknown signal '" + name + "'.") }
  handlers.push([object, handlerName])
}
SignalsMixin.disconnect = function (name, object, handlerName) {
  handlerName = handlerName || ('_on' + name)
  var handlers = this.signals[name]
  if (handlers === undefined) { throw new Error("Unknown signal '" + name + "'.") }
  var size = handlers.length
  for (var i = 0; i < size; i++) {
    var handler = handlers[i]
    if (handler[0] === object && handler[1] === handlerName) {
      handlers.splice(i, 1)
      break
    }
  }
}
SignalsMixin.emit = function (name, varArgsList) {
  var handlers = this.signals[name]
  if (handlers === undefined) { throw new Error("Unknown signal '" + name + "'.") }
  var size = handlers.length
  var args = [this]
  for (var i = 1; i < arguments.length; i++) {
    args.push(arguments[i])
  }
  for (i = 0; i < size; i++) {
    var handler = handlers[i]
    var object = handler[0]
    object[handler[1]].apply(object, args)
  }
  args[0] = name
  return args
}
Nuvola.SignalsMixin = SignalsMixin
// Included file '/builddir/build/BUILD/nuvolaruntime-4.11.0/src/mainjs/actions.js'
var Actions = Nuvola.$prototype(null, Nuvola.SignalsMixin)
Actions.$init = function () {
  this.addSignal('ActionActivated')
  this.addSignal('ActionEnabledChanged')
  this.connect('ActionActivated', this)
  this.connect('ActionEnabledChanged', this)
  this.buttons = {}
  this.enabledFlagsCache = {}
  this.statesCache = {}
}
Actions.addAction = function (group, scope, name, label, mnemoLabel, icon, keybinding, state) {
  if (state === undefined) { state = null }
  Nuvola._callIpcMethodVoid('/nuvola/actions/add-action',
    [group, scope, name, label || '', mnemoLabel || '', icon || '', keybinding || '', state])
}
Actions.addRadioAction = function (group, scope, name, stateId, options) {
  Nuvola._callIpcMethodVoid('/nuvola/actions/add-radio-action', [group, scope, name, stateId, options])
}
Actions._onActionActivated = function (arg1, action) {
  console.log('JS API: Action activated: ' + action)
}
Actions.isEnabled = function (name) {
  Nuvola.log('Actions.isEnabled is deprecated since Nuvola 4.8. Use async variant instead.')
  return Nuvola._callIpcMethodSync('/nuvola/actions/is-enabled', [name])
}
Actions.isEnabledAsync = function (name) {
  return Nuvola.Async.call('/nuvola/actions/is-enabled', [name])
}
Actions.setEnabled = function (name, enabled) {
  Nuvola._callIpcMethodVoid('/nuvola/actions/set-enabled', [name, enabled])
}
Actions.updateEnabledFlag = function (name, enabled) {
  if (enabled !== this.enabledFlagsCache[name]) {
    this.enabledFlagsCache[name] = enabled
    this.setEnabled(name, enabled)
  }
}
Actions.updateEnabledFlags = function (enabledFlags) {
  for (var name in enabledFlags) { this.updateEnabledFlag(name, enabledFlags[name]) }
}
Actions.getState = function (name) {
  Nuvola.log('Actions.getState is deprecated since Nuvola 4.8. Use async variant instead.')
  return Nuvola._callIpcMethodSync('/nuvola/actions/get-state', [name])
}
Actions.getStateAsync = function (name) {
  return Nuvola.Async.call('/nuvola/actions/get-state', [name])
}
Actions.setState = function (name, state) {
  Nuvola._callIpcMethodVoid('/nuvola/actions/set-state', [name, state])
}
Actions.updateState = function (name, state) {
  if (state !== this.statesCache[name]) {
    this.statesCache[name] = state
    this.setState(name, state)
  }
}
Actions.updateStates = function (states) {
  for (var name in states) { this.updateState(name, states[name]) }
}
Actions.activate = function (name, parameter) {
  Nuvola._callIpcMethodVoid('/nuvola/actions/activate', [name, parameter == null ? null : parameter])
}
Actions.bindButton = function (button, name, parameter) {
  this.buttons[name] = button
  this.isEnabledAsync(name).then((enabled) => { button.disabled = !enabled })
  button.addEventListener('click', () => this.activate(name, parameter))
}
Actions._onActionEnabledChanged = function (object, name, enabled) {
  if (this.buttons[name]) { this.buttons[name].disabled = !enabled }
}
Nuvola.Actions = Actions
Nuvola.actions = Nuvola.$object(Actions)
// Included file '/builddir/build/BUILD/nuvolaruntime-4.11.0/src/mainjs/async.js'
var Async = {}
Async.nextPromiseId = 1
Async.promises = {}
Async.MAX_PROMISE_ID = 32767
Async.begin = function (func) {
  var ctx = {}
  while (this.promises[this.nextPromiseId]) {
    if (this.nextPromiseId === this.MAX_PROMISE_ID) {
      this.nextPromiseId = 1
    } else {
      this.nextPromiseId++
    }
  }
  this.promises[this.nextPromiseId] = ctx
  ctx.id = this.nextPromiseId
  ctx.promise = new Promise(function (resolve, reject) { ctx.resolve = resolve; ctx.reject = reject })
  try {
    func(ctx)
  } catch (error) {
    ctx.reject(error)
    this.remove(ctx)
  }
  return ctx.promise
}
Async.remove = function (ctx) {
  delete this.promises[ctx.id]
  delete ctx.promise
  delete ctx.reject
  delete ctx.resolve
}
Async.call = function (path, params) {
  return this.begin((ctx) => Nuvola._callIpcMethodAsync(path, params || null, ctx.id))
}
Async.respond = function (id, response, error) {
  var ctx = this.promises[id]
  if (ctx) {
    if (error) {
      ctx.reject(error)
    } else {
      ctx.resolve(response)
    }
    this.remove(ctx)
  } else {
    throw new Error('Promise ' + id + ' not found.')
  }
}
Nuvola.Async = Async
// Included file '/builddir/build/BUILD/nuvolaruntime-4.11.0/src/mainjs/browser.js'
var BrowserAction = {
  GO_BACK: 'go-back',
  GO_FORWARD: 'go-forward',
  GO_HOME: 'go-home',
  RELOAD: 'reload'
}
var Browser = Nuvola.$prototype(null)
Browser.$init = function () {
  this._downloadFileAsyncId = 0
  this._downloadFileAsyncCallbacks = {}
}
Browser.downloadFileAsync = function (uri, basename, callback, data) {
  var id = this._downloadFileAsyncId++
  if (this._downloadFileAsyncId >= Number.MAX_VALUE - 1) { this._downloadFileAsyncId = 0 }
  this._downloadFileAsyncCallbacks[id] = [callback, data]
  Nuvola._callIpcMethodVoid('/nuvola/browser/download-file-async', [uri, basename, id])
}
Browser._downloadDone = function (id, success, statusCode, statusText, filePath, fileURI) {
  var cb = this._downloadFileAsyncCallbacks[id]
  delete this._downloadFileAsyncCallbacks[id]
  if (cb) {
    cb[0]({
      success: success,
      statusCode: statusCode,
      statusText: statusText,
      filePath: filePath,
      fileURI: fileURI
    }, cb[1])
  }
}
Nuvola.BrowserAction = BrowserAction
Nuvola.Browser = Browser
Nuvola.browser = Nuvola.$object(Browser)
// Included file '/builddir/build/BUILD/nuvolaruntime-4.11.0/src/mainjs/utils.js'
Nuvola.format = function () {
  var args = arguments
  return args[0].replace(Nuvola.format._regex, function (item) {
    var index = parseInt(item.substring(1, item.length - 1))
    if (index > 0) { return typeof args[index] !== 'undefined' ? args[index] : '' } else if (index === -1) { return '{' } else if (index === -2) { return '}' }
    return ''
  })
}
Nuvola.formatVersion = function (encodedVersion) {
  var micro = encodedVersion % 100
  encodedVersion = (encodedVersion - micro) / 100
  var minor = encodedVersion % 100
  var major = (encodedVersion - minor) / 100
  return major + '.' + minor + '.' + micro
}
Nuvola.format._regex = new RegExp('{-?[0-9]+}', 'g')
Nuvola.inArray = function (array, item) {
  return array.indexOf(item) > -1
}
Nuvola.triggerMouseEvent = function (elm, name, x, y) {
  var rect = elm.getBoundingClientRect()
  var width = rect.width * (x === undefined ? 0.5 : x)
  var height = rect.height * (y === undefined ? 0.5 : y)
  var opts = {
    view: document.defaultView,
    bubbles: true,
    cancelable: true,
    button: 0,
    relatedTarget: elm
  }
  opts.clientX = rect.left + width
  opts.clientY = rect.top + height
  opts.screenX = window.screenX + opts.clientX
  opts.screenY = window.screenY + opts.clientY
  var event = new window.MouseEvent(name, opts)
  elm.dispatchEvent(event)
}
Nuvola.clickOnElement = function (elm, x, y) {
  Nuvola.triggerMouseEvent(elm, 'mouseover', x, y)
  Nuvola.triggerMouseEvent(elm, 'mousedown', x, y)
  Nuvola.triggerMouseEvent(elm, 'mouseup', x, y)
  Nuvola.triggerMouseEvent(elm, 'click', x, y)
}
Nuvola.setInputValueWithEvent = function (elm, value) {
  elm.value = value
  elm.dispatchEvent(new window.Event('input', {'bubbles': true, 'cancelable': true}))
}
Nuvola.makeText = function (text) {
  return document.createTextNode(text)
}
Nuvola.makeElement = function (name, attributes, text) {
  var elm = document.createElement(name)
  attributes = attributes || {}
  for (var key in attributes) { elm.setAttribute(key, attributes[key]) }
  if (text !== undefined && text !== null) { elm.appendChild(Nuvola.makeText(text)) }
  return elm
}
Nuvola.objectDiff = function (object1, object2) {
  var changes = []
  for (var property in object1) {
    if (object1.hasOwnProperty(property) &&
        (!object2.hasOwnProperty(property) || object1[property] !== object2[property])) { changes.push(property) }
  }
  return changes
}
Nuvola.parseTimeUsec = function (time) {
  if (!time) { return 0 }
  if (time * 1 === time) { return time }
  var parts = time.split(':')
  var sign = parts[0] * 1 < 0 ? -1 : 1
  var seconds = 0
  var item = parts.pop()
  if (item !== undefined) {
    seconds = Math.abs(1 * item)
    item = parts.pop()
    if (item !== undefined) {
      seconds += Math.abs(60 * item)
      item = parts.pop()
      if (item !== undefined) { seconds += Math.abs(60 * 60 * item) }
    }
  }
  return !isNaN(seconds) ? sign * seconds * 1000 * 1000 : 0
}
Nuvola.encodeVersion = function (major, minor, micro) {
  return (major || 0) * 100000 + (minor || 0) * 1000 + (micro || 0)
}
Nuvola.checkVersion = function (major, minor, micro) {
  var v1 = Nuvola.encodeVersion(major, minor, micro)
  var v2 = Nuvola.encodeVersion(Nuvola.VERSION_MAJOR, Nuvola.VERSION_MINOR, Nuvola.VERSION_MICRO)
  return v2 >= v1
}
Nuvola.queryText = function (selector, func) {
  var elm = document.querySelector(selector)
  var value = elm ? elm.textContent.trim() || null : null
  return (value && func) ? func(value, elm) : value
}
Nuvola.queryAttribute = function (selector, attribute, func) {
  var elm = document.querySelector(selector)
  if (!elm) {
    return null
  }
  var value = elm.getAttribute(attribute)
  return func ? func(value, elm) : value
}
Nuvola.exportImageAsBase64 = function (url, callback) {
  var img = new window.Image()
  img.onload = function () {
    var canvas = document.createElement('canvas')
    var ctx = canvas.getContext('2d')
    canvas.height = img.height
    canvas.width = img.width
    ctx.drawImage(img, 0, 0)
    callback(canvas.toDataURL('image/jpeg', 0.95))
    canvas = null
    this.onerror = null
    this.onload = null
    this.src = ''
  }
  img.onerror = function () {
    callback(null)
    this.onload = null
    this.error = null
    this.src = ''
  }
  img.src = url
}
// Included file '/builddir/build/BUILD/nuvolaruntime-4.11.0/src/mainjs/logging.js'
Nuvola.log = function (template) {
  var args = Array.prototype.slice.call(arguments)
  var message = Nuvola.format.apply(Nuvola, args)
  Nuvola._log(message)
}
Nuvola.warn = function (template) {
  var args = Array.prototype.slice.call(arguments)
  var message = Nuvola.format.apply(Nuvola, args)
  Nuvola._warn(message)
}
Nuvola.logException = function (e) {
  console.log(e)
}
// Included file '/builddir/build/BUILD/nuvolaruntime-4.11.0/src/mainjs/core.js'
var Core = Nuvola.$prototype(null, Nuvola.SignalsMixin)
Core.$init = function () {
  this.addSignal('InitAppRunner')
  this.addSignal('InitializationForm')
  this.addSignal('InitWebWorker')
  this.addSignal('InitWebWorkerHelper')
  this.addSignal('HomePageRequest')
  this.addSignal('NavigationRequest')
  this.addSignal('PageSettings')
  this.addSignal('ResourceRequest')
  this.addSignal('LastPageRequest')
  this.addSignal('QuitRequest')
  this.addSignal('UriChanged')
  this.addSignal('PreferencesForm')
  this.addSignal('ComponentLoaded')
  this.addSignal('ComponentUnloaded')
}
Core.getComponentInfo = function (id) {
  Nuvola.log('Core.getComponentInfo is deprecated since Nuvola 4.8. Use async variant instead.')
  return Nuvola._callIpcMethodSync('/nuvola/core/get-component-info', [id + ''])
}
Core.getComponentInfoAsync = function (id) {
  return Nuvola.Async.call('/nuvola/core/get-component-info', [id + ''])
}
Core.isComponentLoaded = function (id) {
  Nuvola.log('Core.isComponentLoaded is deprecated since Nuvola 4.8. Use async variant instead.')
  var info = this.getComponentInfo(id)
  return info.loaded
}
Core.isComponentLoadedAsync = function (id) {
  return this.getComponentInfoAsync(id).then((info) => info.loaded)
}
Core.toggleComponentActive = function (id, active) {
  Nuvola._callIpcMethodVoid('/nuvola/core/toggle-component-active', [id + '', !!active])
  return true
}
Core.isComponentActive = function (id) {
  Nuvola.log('Core.isComponentActive is deprecated since Nuvola 4.8. Use async variant instead.')
  var info = this.getComponentInfo(id)
  return info.active
}
Core.isComponentActiveAsync = function (id) {
  return this.getComponentInfoAsync(id).then((info) => info.active)
}
Nuvola.Core = Core
Nuvola.core = Nuvola.$object(Core)
// Included file '/builddir/build/BUILD/nuvolaruntime-4.11.0/src/mainjs/fallback.js'
var argsToString = function (args) {
  var strings = []
  for (var i = 0; i < args.length; i++) { strings.push('' + args[i]) }
  return strings.join(' ')
}
if (!global.console) { global.console = {} }
if (!console.log) {
  console.log = function () {
    Nuvola.warn('console.log() is not available, using Nuvola.log as a fallback. The following message might be incomplete.')
    Nuvola.log('{1}', argsToString(arguments))
  }
}
if (!console.debug) {
  console.debug = function () {
    Nuvola.warn('console.debug() is not available, using Nuvola.log as a fallback. The following message might be incomplete.')
    Nuvola.log('{1}', argsToString(arguments))
  }
}
if (!global.alert) {
  global.alert = function () {
    Nuvola.warn('alert() is not available, using Nuvola.log as a fallback. The following message might be incomplete.')
    Nuvola.log('{1}', argsToString(arguments))
  }
}
// Included file '/builddir/build/BUILD/nuvolaruntime-4.11.0/src/mainjs/gettext.js'
var Translate = {}
Translate.gettext = function (text) {
  return text
}
Translate.pgettext = function (context, text) {
  if (!context) {
    Nuvola.warn("Context information passed to Translate.pgettext is invalid: {1}, '{2}'", context, text)
    return Translate.gettext(text)
  }
  return text
}
Translate.ngettext = function (text1, text2, n) {
  return n === 1 ? text1 : text2
}
Nuvola.Translate = Translate
// Included file '/builddir/build/BUILD/nuvolaruntime-4.11.0/src/mainjs/infobars.js'
Nuvola.InfoBarType = {
  INFO: 0,
  WARNING: 1,
  QUESTION: 2,
  ERROR: 3
}
Nuvola.showInfoBar = function (id, type, text) {
  id = id + ''
  if (!id) {
    throw new Error('Id must not be empty.')
  }
  return Nuvola.Async.call('/nuvola/show-info-bar', [id, 1 * type, text + ''])
}
// Included file '/builddir/build/BUILD/nuvolaruntime-4.11.0/src/mainjs/launcher.js'
var Launcher = Nuvola.$prototype(null)
Launcher.setTooltip = function (tooltip) {
  Nuvola._callIpcMethodVoid('/nuvola/launcher/set-tooltip', [tooltip || ''])
}
Launcher.setActions = function (actions) {
  Nuvola._callIpcMethodVoid('/nuvola/launcher/set-actions', [actions])
}
Launcher.removeActions = function () {
  Nuvola._callIpcMethodVoid('/nuvola/launcher/remove-actions')
}
Launcher.addAction = function (action) {
  Nuvola._callIpcMethodVoid('/nuvola/launcher/add-action', [action])
}
Launcher.removeAction = function (action) {
  Nuvola._callIpcMethodVoid('/nuvola/launcher/remove-action', [action])
}
Nuvola.Launcher = Launcher
Nuvola.launcher = Nuvola.$object(Launcher)
// Included file '/builddir/build/BUILD/nuvolaruntime-4.11.0/src/mainjs/mediakeys.js'
var MediaKey = {
  PLAY: 'Play',
  PAUSE: 'Pause',
  STOP: 'Stop',
  PREV: 'Previous',
  NEXT: 'Next'
}
var MediaKeys = Nuvola.$prototype(null, Nuvola.SignalsMixin)
MediaKeys.$init = function () {
  this.addSignal('MediaKeyPressed')
}
Nuvola.MediaKey = MediaKey
Nuvola.MediaKeys = MediaKeys
Nuvola.mediaKeys = Nuvola.$object(MediaKeys)
// Included file '/builddir/build/BUILD/nuvolaruntime-4.11.0/src/mainjs/notification.js'
var Notification = Nuvola.$prototype(null)
Notification.$init = function (name, resident, category) {
  this.name = name
  this.resident = !!resident
  this.category = category || ''
}
Notification.update = function (title, text, iconName, iconPath, resident) {
  if (resident == null) { resident = this.resident } else { this.resident = !!resident }
  Nuvola._callIpcMethodVoid('/nuvola/notification/update',
        [this.name, title, text, iconName || '', iconPath || '', !!resident, this.category])
}
Notification.setActions = function (actions) {
  Nuvola._callIpcMethodVoid('/nuvola/notification/set-actions', [this.name, actions])
}
Notification.removeActions = function () {
  Nuvola._callIpcMethodVoid('/nuvola/notification/remove-actions', [this.name])
}
Notification.show = function (force) {
  Nuvola._callIpcMethodVoid('/nuvola/notification/show', [this.name, !!force])
}
var Notifications = Nuvola.$prototype(null)
Notifications.getNamedNotification = function (name, resident, category) {
  return Nuvola.$object(Notification, name, resident, category)
}
Notifications.isPersistenceSupported = function () {
  Nuvola.log('Notifications.isPersistenceSupported is deprecated since Nuvola 4.8. Use async variant instead.')
  return Nuvola._callIpcMethodSync('/nuvola/notifications/is-persistence-supported')
}
Notifications.isPersistenceSupportedAsync = function () {
  return Nuvola.Async.call('/nuvola/notifications/is-persistence-supported')
}
Notifications.showNotification = function (title, text, iconName, iconPath, force, category) {
  Nuvola._callIpcMethodVoid('/nuvola/notifications/show-notification',
        [title, text || '', iconName || '', iconPath || '', !!force, category || ''])
}
Nuvola.Notification = Notification
Nuvola.Notifications = Notifications
Nuvola.notifications = Nuvola.$object(Notifications)
// Included file '/builddir/build/BUILD/nuvolaruntime-4.11.0/src/mainjs/storage.js'
var KeyValueStorage = Nuvola.$prototype(null)
KeyValueStorage.$init = function (index) {
  this.index = index
}
KeyValueStorage.setDefault = function (key, value) {
  Nuvola._keyValueStorageSetDefaultValue(this.index, key, value)
}
KeyValueStorage.setDefaultAsync = function (key, value) {
  return Nuvola.Async.begin((ctx) => Nuvola._keyValueStorageSetDefaultValueAsync(this.index, key, value, ctx.id))
}
KeyValueStorage.hasKey = function (key) {
  return Nuvola._keyValueStorageHasKey(this.index, key)
}
KeyValueStorage.hasKeyAsync = function (key) {
  return Nuvola.Async.begin((ctx) => Nuvola._keyValueStorageHasKeyAsync(this.index, key, ctx.id))
}
KeyValueStorage.get = function (key) {
  return Nuvola._keyValueStorageGetValue(this.index, key)
}
KeyValueStorage.getAsync = function (key) {
  return Nuvola.Async.begin((ctx) => Nuvola._keyValueStorageGetValueAsync(this.index, key, ctx.id))
}
KeyValueStorage.set = function (key, value) {
  if (Object.prototype.toString.call(value) === '[object Object]') {
    throw new Error('Key-value storage is for primitive types only. It is not yet possible to store objects.')
  }
  Nuvola._keyValueStorageSetValue(this.index, key, value)
}
KeyValueStorage.setAsync = function (key, value) {
  return Nuvola.Async.begin((ctx) => {
    if (Object.prototype.toString.call(value) === '[object Object]') {
      throw new Error('Key-value storage is for primitive types only. It is not yet possible to store objects.')
    } else {
      Nuvola._keyValueStorageSetValueAsync(this.index, key, value, ctx.id)
    }
  })
}
var ConfigStorage = Nuvola.$prototype(KeyValueStorage, Nuvola.SignalsMixin)
ConfigStorage.$init = function () {
  KeyValueStorage.$init.call(this, 0)
  this.addSignal('ConfigChanged')
}
var SessionStorage = Nuvola.$prototype(KeyValueStorage)
SessionStorage.$init = function () {
  KeyValueStorage.$init.call(this, 1)
}
Nuvola.KeyValueStorage = KeyValueStorage
Nuvola.ConfigStorage = ConfigStorage
Nuvola.SessionStorage = SessionStorage
Nuvola.session = Nuvola.$object(SessionStorage)
Nuvola.config = Nuvola.$object(ConfigStorage)
// Included file '/builddir/build/BUILD/nuvolaruntime-4.11.0/src/mainjs/mediaplayer.js'
var _ = Nuvola.Translate.gettext
var C_ = Nuvola.Translate.pgettext
var PlayerAction = {
  PLAY: 'play',
  TOGGLE_PLAY: 'toggle-play',
  PAUSE: 'pause',
  STOP: 'stop',
  PREV_SONG: 'prev-song',
  NEXT_SONG: 'next-song',
  PLAYBACK_NOTIFICATION: 'playback-notification',
  SEEK: 'seek',
  CHANGE_VOLUME: 'change-volume'
}
var PlaybackState = {
  UNKNOWN: 0,
  PAUSED: 1,
  PLAYING: 2
}
var RUN_IN_BACKGROUND = 'player.run_in_background'
var BACKGROUND_PLAYBACK = 'player.background_playback'
var RUN_IN_BACKGROUND_OPTIONS = [
    ['always', C_('Background playback', 'Always')],
    ['playing', C_('Background playback', 'When song is playing')],
    ['never', C_('Background playback', 'Never')]
]
var COMPONENT_NOTIFICATIONS = 'notifications'
var COMPONENTS_TO_ACTIVATE = ['mpris', 'lyrics', 'mediakeys', 'scrobbler']
var MediaPlayer = Nuvola.$prototype(null, Nuvola.SignalsMixin)
MediaPlayer.$init = function () {
  Nuvola.mediaPlayer = this
  this.addSignal('RatingSet')
  this._appRunner = false
  this._state = null
  this._artworkFile = null
  this._canGoPrev = null
  this._canGoNext = null
  this._canPlay = null
  this._canPause = null
  this._canRate = null
  this._canSeek = null
  this._canChangeVolume = null
  this._extraActions = []
  this._artworkLoop = 0
  this._baseActions = [PlayerAction.TOGGLE_PLAY, PlayerAction.PLAY, PlayerAction.PAUSE, PlayerAction.PREV_SONG, PlayerAction.NEXT_SONG]
  this._notification = null
  this._trackPosition = 0
  this._volume = 1.0
  Nuvola.core.connect('InitAppRunner', this)
  Nuvola.core.connect('InitWebWorker', this)
}
MediaPlayer.setTrack = function (track) {
  track.length = Nuvola.parseTimeUsec(track.length)
  var changed = Nuvola.objectDiff(this._track, track)
  if (!changed.length) { return }
  this._track = Object.assign({}, track)
  if (!track.artLocation) { this._artworkFile = null }
  if (Nuvola.inArray(changed, 'artLocation') && track.artLocation) {
    this._artworkFile = null
    var artworkId = this._artworkLoop++
    if (this._artworkLoop > 9) { this._artworkLoop = 0 }
    Nuvola.browser.downloadFileAsync(track.artLocation, 'player.artwork.' + artworkId, this._onArtworkDownloaded.bind(this), changed)
    this._sendDevelInfo()
  } else {
    this._updateTrackInfo(changed)
  }
}
MediaPlayer.setTrackPosition = function (position) {
  position = Nuvola.parseTimeUsec(position)
  if (this._trackPosition !== position) {
    this._trackPosition = position
    Nuvola._callIpcMethodVoid('/nuvola/mediaplayer/set-track-position', [position])
  }
}
MediaPlayer.updateVolume = function (volume) {
  volume = (volume || 0.0) * 1
  if (this._volume !== volume) {
    this._volume = volume
    Nuvola._callIpcMethodVoid('/nuvola/mediaplayer/update-volume', [volume])
  }
}
MediaPlayer.setPlaybackState = function (state) {
  if (this._state !== state) {
    this._state = state
    Nuvola.actions.setEnabled(PlayerAction.PLAYBACK_NOTIFICATION, this._state !== PlaybackState.UNKNOWN)
    this._setActions()
    this._updateTrackInfo(['state'])
  }
}
MediaPlayer._setFlag = function (name, state) {
  Nuvola._callIpcMethodVoid('/nuvola/mediaplayer/set-flag', [name, state])
}
MediaPlayer.setCanGoNext = function (canGoNext) {
  if (this._canGoNext !== canGoNext) {
    this._canGoNext = canGoNext
    Nuvola.actions.setEnabled(PlayerAction.NEXT_SONG, !!canGoNext)
    this._setFlag('can-go-next', !!canGoNext)
    this._showNotification()
    this._sendDevelInfo()
  }
}
MediaPlayer.setCanGoPrev = function (canGoPrev) {
  if (this._canGoPrev !== canGoPrev) {
    this._canGoPrev = canGoPrev
    Nuvola.actions.setEnabled(PlayerAction.PREV_SONG, !!canGoPrev)
    this._setFlag('can-go-previous', !!canGoPrev)
    this._showNotification()
    this._sendDevelInfo()
  }
}
MediaPlayer.setCanPlay = function (canPlay) {
  if (this._canPlay !== canPlay) {
    this._canPlay = canPlay
    Nuvola.actions.setEnabled(PlayerAction.PLAY, !!canPlay)
    Nuvola.actions.setEnabled(PlayerAction.TOGGLE_PLAY, !!(this._canPlay || this._canPause))
    this._setFlag('can-play', !!canPlay)
    this._showNotification()
    this._sendDevelInfo()
  }
}
MediaPlayer.setCanPause = function (canPause) {
  if (this._canPause !== canPause) {
    this._canPause = canPause
    Nuvola.actions.setEnabled(PlayerAction.PAUSE, !!canPause)
    Nuvola.actions.setEnabled(PlayerAction.STOP, !!canPause)
    Nuvola.actions.setEnabled(PlayerAction.TOGGLE_PLAY, !!(this._canPlay || this._canPause))
    this._setFlag('can-pause', !!canPause)
    this._setFlag('can-stop', !!canPause)
    this._showNotification()
    this._sendDevelInfo()
  }
}
MediaPlayer.setCanRate = function (canRate) {
  if (this._canRate !== canRate) {
    this._canRate = canRate
    this._setFlag('can-rate', !!canRate)
  }
}
MediaPlayer.setCanSeek = function (canSeek) {
  if (this._canSeek !== canSeek) {
    this._canSeek = canSeek
    Nuvola.actions.setEnabled(PlayerAction.SEEK, !!canSeek)
    this._setFlag('can-seek', !!canSeek)
  }
}
MediaPlayer.setCanChangeVolume = function (canChangeVolume) {
  if (this._canChangeVolume !== canChangeVolume) {
    this._canChangeVolume = canChangeVolume
    Nuvola.actions.setEnabled(PlayerAction.CHANGE_VOLUME, !!canChangeVolume)
    this._setFlag('can-change-volume', !!canChangeVolume)
  }
}
MediaPlayer.addExtraActions = function (actions) {
  var update = false
  for (var i = 0; i < actions.length; i++) {
    var action = actions[i]
    if (!Nuvola.inArray(this._extraActions, action)) {
      this._extraActions.push(action)
      update = true
    }
  }
  if (update) {
    this._updateMenu()
    this._setActions()
  }
}
MediaPlayer._onInitAppRunner = function (emitter) {
  this._appRunner = true
  Nuvola.launcher.setActions(['quit'])
  Nuvola.actions.addAction('playback', 'win', PlayerAction.PLAY, 'Play', null, 'media-playback-start', null)
  Nuvola.actions.addAction('playback', 'win', PlayerAction.PAUSE, 'Pause', null, 'media-playback-pause', null)
  Nuvola.actions.addAction('playback', 'win', PlayerAction.TOGGLE_PLAY, 'Toggle play/pause', null, null, null)
  Nuvola.actions.addAction('playback', 'win', PlayerAction.STOP, 'Stop', null, 'media-playback-stop', null)
  Nuvola.actions.addAction('playback', 'win', PlayerAction.PREV_SONG, 'Previous song', null, 'media-skip-backward', null)
  Nuvola.actions.addAction('playback', 'win', PlayerAction.NEXT_SONG, 'Next song', null, 'media-skip-forward', null)
  Nuvola.actions.addAction('playback', 'win', PlayerAction.SEEK, 'Seek', null, null, null, 0)
  Nuvola.actions.addAction('playback', 'win', PlayerAction.CHANGE_VOLUME, 'Change volume', null, null, null, -1.0)
  Nuvola.actions.addAction('playback', 'win', PlayerAction.PLAYBACK_NOTIFICATION, 'Show playback notification', null, null, null)
  Nuvola.core.connect('ComponentLoaded', this)
  Nuvola.core.connect('ComponentUnloaded', this)
  Nuvola.core.isComponentLoadedAsync(COMPONENT_NOTIFICATIONS).then((loaded) => this._toggleNotifications(loaded))
  for (var i = 0; i < COMPONENTS_TO_ACTIVATE.length; i++) {
    this._activateComponent(COMPONENTS_TO_ACTIVATE[i])
  }
  Nuvola.config.setDefaultAsync(BACKGROUND_PLAYBACK, true)
        .then(() => Nuvola.config.getAsync(BACKGROUND_PLAYBACK))
        .then((enabled) => {
          var defaultOption = RUN_IN_BACKGROUND_OPTIONS[enabled ? 1 : 2][0]
          return Nuvola.config.setDefaultAsync(RUN_IN_BACKGROUND, defaultOption)
        }).catch(console.log.bind(console))
  this._updateMenu()
  Nuvola.core.connect('PreferencesForm', this)
}
MediaPlayer._onInitWebWorker = function (emitter) {
  Nuvola.mediaKeys.connect('MediaKeyPressed', this)
  Nuvola.actions.connect('ActionActivated', this)
  Nuvola.core.connect('QuitRequest', this)
  this._track = {
    'title': undefined,
    'artist': undefined,
    'album': undefined,
    'artLocation': undefined,
    'rating': undefined
  }
  this.setPlaybackState(PlaybackState.UNKNOWN)
  this._setActions()
  Nuvola.core.connect('ComponentLoaded', this)
  Nuvola.core.connect('ComponentUnloaded', this)
  Nuvola.core.isComponentLoadedAsync(COMPONENT_NOTIFICATIONS).then((loaded) => this._toggleNotifications(loaded))
  Nuvola.config.connect('ConfigChanged', this)
  Nuvola.config.getAsync(RUN_IN_BACKGROUND)
        .then((value) => { this._runInBackground = value })
        .catch(console.log.bind(console))
}
MediaPlayer._onActionActivated = function (emitter, name, param) {
  switch (name) {
    case PlayerAction.PLAYBACK_NOTIFICATION:
      if (this._notification) { this._notification.show() }
      break
  }
}
MediaPlayer._setActions = function () {
  var actions = [this._state === PlaybackState.PLAYING ? PlayerAction.PAUSE : PlayerAction.PLAY, PlayerAction.PREV_SONG, PlayerAction.NEXT_SONG]
  actions = actions.concat(this._extraActions)
  actions.push('quit')
  Nuvola.launcher.setActions(actions)
}
MediaPlayer._sendDevelInfo = function () {
  var rating = 1 * this._track.rating
  if (rating < 0 || isNaN(rating)) { rating = 0.0 } else if (rating > 1) { rating = 1 }
  var info = {
    'title': this._track.title || null,
    'artist': this._track.artist || null,
    'album': this._track.album || null,
    'rating': rating,
    'length': this._track.length || 0,
    'artworkLocation': this._track.artLocation || null,
    'artworkFile': this._artworkFile || null,
    'playbackActions': this._baseActions.concat(this._extraActions),
    'state': ['unknown', 'paused', 'playing'][this._state]
  }
  Nuvola._callIpcMethodVoid('/nuvola/mediaplayer/set-track-info', info)
}
MediaPlayer._onArtworkDownloaded = function (res, changed) {
  if (!res.success) {
    this._artworkFile = null
    console.log(Nuvola.format('Artwork download failed: {1} {2}.', res.statusCode, res.statusText))
  } else {
    this._artworkFile = res.filePath
  }
  this._updateTrackInfo(changed)
}
MediaPlayer._updateTrackInfo = function (changed) {
  this._sendDevelInfo()
  var track = this._track
  if (track.title) {
    this._updateNotification()
    var tooltip = track.artist ? Nuvola.format('{1} by {2}', track.title, track.artist) : track.title
    Nuvola.launcher.setTooltip(tooltip)
  } else {
    Nuvola.launcher.setTooltip('Nuvola Player')
  }
}
MediaPlayer._updateNotification = function () {
  var track = this._track
  if (this._notification && track.title) {
    var title = track.title
    var message
    if (!track.artist && !track.album) { message = 'by unknown artist' } else if (!track.artist) { message = Nuvola.format('from {1}', track.album) } else if (!track.album) { message = Nuvola.format('by {1}', track.artist) } else { message = Nuvola.format('by {1} from {2}', track.artist, track.album) }
    this._notification.update(title, message, this._artworkFile ? null : 'nuvolaplayer', this._artworkFile)
    this._showNotification()
  }
}
MediaPlayer._showNotification = function () {
  if (this._notification) {
    Nuvola.notifications.isPersistenceSupportedAsync().then((supported) => {
      if (this._state === PlaybackState.PLAYING || supported) {
        this._notification.show()
      }
    })
  }
}
MediaPlayer._updateMenu = function () {
  Nuvola.menuBar.setMenu('playback', '_Control', this._baseActions.concat(this._extraActions))
}
MediaPlayer._onQuitRequest = function (emitter, result) {
  var option = this._runInBackground
  if (option === RUN_IN_BACKGROUND_OPTIONS[0][0] ||
    (option === RUN_IN_BACKGROUND_OPTIONS[1][0] && this._state === PlaybackState.PLAYING)) {
    result.approved = false
  }
}
MediaPlayer._onPreferencesForm = function (object, values, entries) {
  values[RUN_IN_BACKGROUND] = Nuvola.config.get(RUN_IN_BACKGROUND)
  entries.push(['label', _('Run in background when window is closed')])
  for (var i = 0; i < RUN_IN_BACKGROUND_OPTIONS.length; i++) {
    var option = RUN_IN_BACKGROUND_OPTIONS[i]
    entries.push(['option', RUN_IN_BACKGROUND, option[0], option[1]])
  }
}
MediaPlayer._onMediaKeyPressed = function (emitter, key) {
  var A = Nuvola.actions
  switch (key) {
    case Nuvola.MediaKey.PLAY:
    case Nuvola.MediaKey.PAUSE:
      A.activate(PlayerAction.TOGGLE_PLAY)
      break
    case Nuvola.MediaKey.STOP:
      A.activate(PlayerAction.STOP)
      break
    case Nuvola.MediaKey.NEXT:
      A.activate(PlayerAction.NEXT_SONG)
      break
    case Nuvola.MediaKey.PREV:
      A.activate(PlayerAction.PREV_SONG)
      break
    default:
      console.log(Nuvola.format("Unknown media key '{1}'.", key))
      break
  }
}
MediaPlayer._onComponentLoaded = function (emitter, id) {
  if (id === COMPONENT_NOTIFICATIONS) { this._toggleNotifications(true) }
  if (Nuvola.inArray(COMPONENTS_TO_ACTIVATE, id)) { this._activateComponent(id) }
}
MediaPlayer._onComponentUnloaded = function (emitter, id) {
  if (id === COMPONENT_NOTIFICATIONS) { this._toggleNotifications(false) }
}
MediaPlayer._activateComponent = function (id) {
  Nuvola.core.toggleComponentActive(id, true)
}
MediaPlayer._toggleNotifications = function (enabled) {
  if (enabled) {
    this._notification = Nuvola.notifications.getNamedNotification('mediaplayer', true, 'x-gnome.music')
    if (this._appRunner) { this._notification.setActions([PlayerAction.PREV_SONG, PlayerAction.PLAY, PlayerAction.PAUSE, PlayerAction.NEXT_SONG]) } else { this._updateNotification() }
  } else {
    this._notification = null
  }
}
MediaPlayer._onConfigChanged = function (emitter, key) {
  if (key === RUN_IN_BACKGROUND) {
    Nuvola.config.getAsync(RUN_IN_BACKGROUND)
        .then((value) => { this._runInBackground = value })
        .catch(console.log.bind(console))
  }
}
Nuvola.PlayerAction = PlayerAction
Nuvola.PlaybackState = PlaybackState
Nuvola.MediaPlayer = MediaPlayer
// Included file '/builddir/build/BUILD/nuvolaruntime-4.11.0/src/mainjs/menubar.js'
var MenuBar = Nuvola.$prototype(null)
MenuBar.setMenu = function (id, label, actions) {
  Nuvola._callIpcMethodVoid('/nuvola/menubar/set-menu', [id, label, actions])
}
Nuvola.MenuBar = MenuBar
Nuvola.menuBar = Nuvola.$object(MenuBar)
// Included file '/builddir/build/BUILD/nuvolaruntime-4.11.0/src/mainjs/webapp.js'
var WebApp = Nuvola.$prototype(null, Nuvola.SignalsMixin)
WebApp.LAST_URI = 'web_app.last_uri'
WebApp.$init = function () {
  this.meta = Nuvola.meta
  var allowedURI = this.meta.allowed_uri
  this.allowedURI = allowedURI ? new RegExp(allowedURI) : null
  Nuvola.core.connect('HomePageRequest', this)
  Nuvola.core.connect('LastPageRequest', this)
  Nuvola.core.connect('NavigationRequest', this)
  Nuvola.core.connect('UriChanged', this)
  Nuvola.core.connect('InitAppRunner', this)
  Nuvola.core.connect('InitWebWorker', this)
  Nuvola.core.connect('InitWebWorkerHelper', this)
}
WebApp.start = function () {
  Nuvola.webApp = Nuvola.$object(this)
}
WebApp._onHomePageRequest = function (emitter, result) {
  result.url = this.meta.home_url
}
WebApp._onLastPageRequest = function (emitter, request) {
  request.url = Nuvola.config.get(this.LAST_URI) || null
}
WebApp._onNavigationRequest = function (object, request) {
  request.approved = this.allowedURI ? this.allowedURI.test(request.url) : true
}
WebApp._onUriChanged = function (object, uri) {
  Nuvola.config.setAsync(this.LAST_URI, uri).catch(console.log.bind(console))
}
WebApp._onInitAppRunner = function (emitter) {
}
WebApp._onInitWebWorker = function (emitter) {
}
WebApp._onInitWebWorkerHelper = function (emitter) {
}
Nuvola.WebApp = WebApp
Nuvola.$WebApp = function () {
  if (this !== Nuvola) { throw new Error('Nuvola.$WebApp has been called incorrectly. Use `var WebApp = Nuvola.$WebApp();` idiom.') }
  return Nuvola.$prototype(WebApp)
}
})(this);  // function(Nuvola)
